/**
 * \file color.cpp
 * \brief class to manage color values
 * \author Luis Alvarez \n \n
*/


#include "color.h"
#include "stdio.h"

/// CONVERT HSV TO RGB
void color::hsv2rgb(float h,float s,float v){
  float      hh, p, q, t, ff;
  long        i;
  float      r,g,b;

  if(s <= 0.0) {       // < is bogus, just shuts up warnings
    r = v;
    g = v;
    b = v;
  }
  else{
    hh = h;
    if(hh >= 360.0) hh = 0.0;
    hh /= 60.0;
    i = (long)hh;
    ff = hh - i;
    p = round(v * (1.0 - s));
    q = round(v * (1.0 - (s * ff)));
    t = round(v * (1.0 - (s * (1.0 - ff))));

    switch(i) {
    case 0:
        r = v;
        g = t;
        b = p;
        break;
    case 1:
        r = q;
        g = v;
        b = p;
        break;
    case 2:
        r = p;
        g = v;
        b = t;
        break;

    case 3:
        r = p;
        g = q;
        b = v;
        break;
    case 4:
        r = t;
        g = p;
        b = v;
        break;
    case 5:
    default:
        r = v;
        g = p;
        b = q;
        break;
    }
  }

  //R=round(R*255.); G=round(255.*G); B=round(255.*B);
  R_ = (unsigned char) (r<0?0:(r>=255?255:r));
  G_ = (unsigned char) (g<0?0:(g>=255?255:g));
  B_ = (unsigned char) (b<0?0:(b>=255?255:b));
  //if(R_==0 && G_==0 && B_==0){printf("i=%d, h=%f s=%f v=%f\n",i,h,s,v); system("pause"); }
}

// CONVERT RGB TO HSV
void color::rgb2hsv(float &H,float &S,float &V){
  float rgb_min,rgb_max;
  float R=R_,G=G_,B=B_;
  rgb_min=rgb_max=R;
  if(rgb_min>G) rgb_min=G;
  if(rgb_min>B) rgb_min=B;
  if(rgb_max<G) rgb_max=G;
  if(rgb_max<B) rgb_max=B;

  V = rgb_max;                                // v
  float delta = rgb_max - rgb_min;
  if (delta < 0.00001)
  {
     S = 0;
     H = 0; // undefined, maybe nan?
    return;
  }
  if( rgb_max > 0.0 ) { // NOTE: if Max is == 0, this divide would cause a crash
    S = (delta / rgb_max);                  // s
  }
  else {
    S = 0.0;
    H = 0.;                            // its now undefined
    return;
  }
  if( R_ == rgb_max )                           // > is bogus, just keeps compilor happy
    H = ( G - B ) / delta;        // between yellow & magenta
  else if( G == rgb_max )
    H = 2.0 + (  B - R ) / delta;  // between cyan & yellow
  else
    H = 4.0 + (  R - G ) / delta;  // between magenta & cyan

  H *= 60.0;                              // degrees

  if( H < 0.0 )  H += 360.0;
}

color min(color &c1,color &c2){
  if( ( (int) c1.R_-c2.R_+c1.G_-c2.G_+c1.B_-c2.B_) <0 )
    return c1;
  return c2;
}

color max(color &c1,color &c2){
  if( ( (int) c1.R_-c2.R_+c1.G_-c2.G_+c1.B_-c2.B_) <0 )
    return c2;
  return c1;
}

color min(color &c1,color &c2,color &c3){
   if( ( (int) c1.R_-c2.R_+c1.G_-c2.G_+c1.B_-c2.B_) <0 ){
     if( ( (int) c1.R_-c3.R_+c1.G_-c3.G_+c1.B_-c3.B_) <0 ){
        return c1;
     }
     return c3;
   }
   else{
     if( ( (int) c2.R_-c3.R_+c2.G_-c3.G_+c2.B_-c3.B_) <0 ){
        return c2;
     }
     return c3;
   }
}

color max(color &c1,color &c2,color &c3){
   if( ( (int) c1.R_-c2.R_+c1.G_-c2.G_+c1.B_-c2.B_) >0 ){
     if( ( (int) c1.R_-c3.R_+c1.G_-c3.G_+c1.B_-c3.B_) >0 ){
        return c1;
     }
     return c3;
   }
   else{
     if( ( (int) c2.R_-c3.R_+c2.G_-c3.G_+c2.B_-c3.B_) >0 ){
        return c2;
     }
     return c3;
   }
}
